import bs
import bsUtils
from bsVector import Vector
import random
import weakref

_gFactory = None

class Factory(object):
    """ wraps up media an other resources used by bombs """

    def getRandomExplodeSound(self):
        return self.explodeSounds[random.randrange(len(self.explodeSounds))]

    def __init__(self):

        self.bombModel = bs.getModel("bomb")
        self.stickyBombModel = bs.getModel("bombSticky")
        self.impactBombModel = bs.getModel("impactBomb")
        self.landMineModel = bs.getModel("landMine")
        self.tntModel = bs.getModel("tnt")

        self.regularTex = bs.getTexture("bombColor")
        self.iceTex = bs.getTexture("bombColorIce")
        self.stickyTex = bs.getTexture("bombStickyColor")
        self.impactTex = bs.getTexture("impactBombColor")
        self.impactLitTex = bs.getTexture("impactBombColorLit")
        self.landMineTex = bs.getTexture("landMine")
        self.landMineLitTex = bs.getTexture("landMineLit")
        self.tntTex = bs.getTexture("tnt")

        self.hissSound = bs.getSound("hiss")
        self.debrisFallSound = bs.getSound("debrisFall")
        self.woodDebrisFallSound = bs.getSound("woodDebrisFall")

        self.explodeSounds = (bs.getSound("explosion01"),
                              bs.getSound("explosion02"),
                              bs.getSound("explosion03"),
                              bs.getSound("explosion04"),
                              bs.getSound("explosion05"))

        self.freezeSound = bs.getSound("freeze")
        self.fuseSound = bs.getSound("fuse01")
        self.activateSound = bs.getSound("activateBeep")
        self.warnSound = bs.getSound("warnBeep")

        # set up our material so new bombs dont collide with objects
        # that they are initially overlapping
        self.bombMaterial = bs._newMaterial()
        self.normalSoundMaterial = bs._newMaterial()
        self.stickyMaterial = bs._newMaterial()

        bs._materialAddAction(self.bombMaterial,
                             conditions=((("srcYoungerThan",100),'or',("dstYoungerThan",100)),'and',
                                         ("dstIsMaterial",bs.objectMaterial)),
                             actions=(("nodeMod","collide",False)))

        # we want pickup materials to always hit us even if we're currently not
        # colliding with their node (generally due to the above rule)
        bs._materialAddAction(self.bombMaterial,
                             conditions=("dstIsMaterial",bs.pickupMaterial),
                             actions=(("partMod","useNodeCollide",False)))
        bs._materialAddAction(self.bombMaterial,actions=("partMod","friction",0.3))

        self.landMineMaterial = bs._newMaterial()
        self.landMineBlastMaterial = bs._newMaterial()
        bs._materialAddAction(self.landMineBlastMaterial,
                             conditions=(("srcOlderThan",200),'and',
                                         ("dstOlderThan",200),'and',
                                         ("evalColliding",),'and',
                                         (("dstNotMaterial",self.landMineMaterial),'and',(("dstIsMaterial",bs.objectMaterial),"or",("dstIsMaterial",bs.playerMaterial)))),
                             actions=(("message","self","atConnect",ImpactMessage())))

        self.impactBlastMaterial = bs._newMaterial()
        bs._materialAddAction(self.impactBlastMaterial,
                             conditions=(("srcOlderThan",200),'and',
                                         ("dstOlderThan",200),'and',
                                         ("evalColliding",),'and',
                                         (("dstIsMaterial",bs.footingMaterial),
                                          'or',("dstIsMaterial",bs.objectMaterial))),
                             actions=(("message","self","atConnect",ImpactMessage())))

        self.blastMaterial = bs._newMaterial()
        bs._materialAddAction(self.blastMaterial,
                             conditions=(("dstIsMaterial",bs.objectMaterial)),
                             actions=(("partMod","collide",True),
                                      ("partMod","physical",False),
                                      ("message","self","atConnect",ExplodeHitMessage())))

        self.dinkSounds = (bs.getSound("bombDrop01"),
                           bs.getSound("bombDrop02"))
        self.stickyImpactSound = bs.getSound("stickyImpact")

        self.rollSound = bs.getSound("bombRoll01")

        # collision sounds
        bs._materialAddAction(self.normalSoundMaterial,
                             conditions=("dstIsMaterial",bs.footingMaterial),
                             actions=(("impactSound",self.dinkSounds,2,0.8),
                                      ("rollSound",self.rollSound,3,6)))

        bs._materialAddAction(self.stickyMaterial,
                             actions=(('partMod','stiffness',0.1),
                                      ('partMod','damping',1.0)))

        bs._materialAddAction(self.stickyMaterial,
                             conditions=(("dstIsMaterial",bs.playerMaterial),"or",("dstIsMaterial",bs.footingMaterial)),
                             actions=(("message","self","atConnect",SplatMessage())))


def getFactory():
    """ return the global factory object, initializing it if necessary """
    global _gFactory
    if _gFactory == None:
        _gFactory = Factory()
    return _gFactory

class SplatMessage(object):
    pass

class ExplodeMessage(object):
    pass

class ImpactMessage(object):
    """ impact bomb touched something """
    pass

class ArmMessage(object):
    pass

class WarnMessage(object):
    pass

class ExplodeHitMessage(object):
    "Message saying an object was hit"
    def __init__(self):
        pass

class Blast(bs.Actor):
    def __init__(self,activity,retain,translate=(0,1,0),velocity=(0,0,0),blastRadius=2.0,type="normal",source=None,hitType='explosion',hitSubType='normal'):
        bs.Actor.__init__(self,activity,retain)

        factory = getFactory()

        self.type = type
        self.source = source

        self.hitType = hitType;
        self.hitSubType = hitSubType;

        # blast radius
        self.radius = blastRadius
        
        self.node = bs.newNode("region",
                               owner=activity,
                               attrs={"translate":(translate[0],translate[1]-0.1,translate[2]), # move down a bit so we throw more stuff upward
                                      "scale":(self.radius,self.radius,self.radius),
                                      "type":"sphere",
                                      "materials":(factory.blastMaterial,bs.attackMaterial)},
                               delegate=self)

        bs.gameTimer(50,self.node.delete)

        # throw in an explosion and flash
        explosion = bs.newNode("explosion",
                               owner=activity,
                               attrs={'translate':translate,
                                      'velocity':(velocity[0],max(-1.0,velocity[1]),velocity[2]),
                                      'radius':self.radius,
                                      'big':(self.type == 'tnt')})
        if self.type == "ice":
            explosion.color = (0,0.05,0.4)

        bs.gameTimer(1000,explosion.delete)

        if self.type != 'ice': bs.emitBGDynamics(position=translate,velocity=velocity,count=int(1.0+random.random()*4),emitType='tendrils',tendrilType='thinSmoke')
        bs.emitBGDynamics(position=translate,velocity=velocity,count=int(4.0+random.random()*4),emitType='tendrils',tendrilType='ice' if self.type == 'ice' else 'smoke')
        bs.emitBGDynamics(position=translate,emitType='distortion',spread=1.0 if self.type == 'tnt' else 2.0)

        # and emit some shrapnel..
        if self.type == 'ice':
            def _doEmit():
                bs.emitBGDynamics(position=translate,velocity=velocity,count=30,spread=2.0,scale=0.4,chunkType='ice',emitType='stickers');
            bs.gameTimer(50,_doEmit) # looks better if we delay a bit


        elif self.type == 'sticky':
            def _doEmit():
                bs.emitBGDynamics(position=translate,velocity=velocity,count=int(4.0+random.random()*8),spread=0.7,chunkType='slime');
                bs.emitBGDynamics(position=translate,velocity=velocity,count=int(4.0+random.random()*8),scale=0.5, spread=0.7,chunkType='slime');
                bs.emitBGDynamics(position=translate,velocity=velocity,count=15,scale=0.6,chunkType='slime',emitType='stickers');
                bs.emitBGDynamics(position=translate,velocity=velocity,count=20,scale=0.7,chunkType='spark',emitType='stickers');
                bs.emitBGDynamics(position=translate,velocity=velocity,count=int(6.0+random.random()*12),scale=0.8,spread=1.5,chunkType='spark');
            bs.gameTimer(50,_doEmit) # looks better if we delay a bit

        elif self.type == 'impact': # regular bomb shrapnel
            def _doEmit():
                bs.emitBGDynamics(position=translate,velocity=velocity,count=int(4.0+random.random()*8),scale=0.8,chunkType='metal');
                bs.emitBGDynamics(position=translate,velocity=velocity,count=int(4.0+random.random()*8),scale=0.4,chunkType='metal');
                bs.emitBGDynamics(position=translate,velocity=velocity,count=20,scale=0.7,chunkType='spark',emitType='stickers');
                bs.emitBGDynamics(position=translate,velocity=velocity,count=int(8.0+random.random()*15),scale=0.8,spread=1.5,chunkType='spark');
            bs.gameTimer(50,_doEmit) # looks better if we delay a bit

        else: # regular or land mine bomb shrapnel
            def _doEmit():
                if self.type != 'tnt':
                    bs.emitBGDynamics(position=translate,velocity=velocity,count=int(4.0+random.random()*8),chunkType='rock');
                    bs.emitBGDynamics(position=translate,velocity=velocity,count=int(4.0+random.random()*8),scale=0.5,chunkType='rock');
                bs.emitBGDynamics(position=translate,velocity=velocity,count=30,scale=1.0 if self.type=='tnt' else 0.7,chunkType='spark',emitType='stickers');
                bs.emitBGDynamics(position=translate,velocity=velocity,count=int(18.0+random.random()*20),scale=1.0 if self.type == 'tnt' else 0.8,spread=1.5,chunkType='spark');

                # tnt throws splintery chunks
                if self.type == 'tnt':
                    def _emitSplinters():
                        bs.emitBGDynamics(position=translate,velocity=velocity,count=int(20.0+random.random()*25),scale=0.8,spread=1.0,chunkType='splinter');
                    bs.gameTimer(10,_emitSplinters)
                
                # every now and then do a sparky one
                if self.type == 'tnt' or random.random() < 0.1:
                    def _emitExtraSparks():
                        bs.emitBGDynamics(position=translate,velocity=velocity,count=int(10.0+random.random()*20),scale=0.8,spread=1.5,chunkType='spark');
                    bs.gameTimer(20,_emitExtraSparks)
                        
            bs.gameTimer(50,_doEmit) # looks better if we delay a bit

        light = bs.newNode("light",
                           owner=activity,
                           attrs={"translate":translate,
                                  "color": (0.6,0.6,1.0) if self.type == 'ice' else (1,0.3,0.1),
                                  "volumeIntensityScale": 10.0})

        s = random.uniform(0.6,0.9)
        scorchRadius = lightRadius = self.radius
        if self.type == 'tnt':
            lightRadius *= 1.4
            scorchRadius *= 1.15
            s *= 3.0

        iScale = 1.6
        bsUtils.animate(light,"intensity",{0:2.0*iScale, int(s*20):0.1*iScale, int(s*25):0.2*iScale, int(s*50):17.0*iScale, int(s*60):5.0*iScale, int(s*80):4.0*iScale, int(s*200):0.6*iScale, int(s*2000):0.00*iScale, int(s*3000):0.0})
        bsUtils.animate(light,"radius",{0:lightRadius*0.2, int(s*50):lightRadius*0.55, int(s*100):lightRadius*0.3, int(s*300):lightRadius*0.15, int(s*1000):lightRadius*0.05})
        bs.gameTimer(int(s*3000),light.delete)

        # make a scorch that fades over time
        scorch = bs.newNode("scorch",
                            owner=activity,
                            attrs={"translate":translate,"size":scorchRadius*0.5,"big":(self.type == 'tnt')})
        if self.type == "ice":
            scorch.color = (1,1,1.5)

        bsUtils.animate(scorch,"presence",{3000:1, 13000:0})
        bs.gameTimer(13000,scorch.delete)

        if self.type == 'ice':
            bs.playSound(factory.hissSound,position=(light,"translate"))
            
        p = light.translate
        bs.playSound(factory.getRandomExplodeSound(),position=p)
        bs.playSound(factory.debrisFallSound,position=p)

        bs.cameraShake(intensity=5.0 if self.type == 'tnt' else 1.0)

        # tnt is more epic..
        if self.type == 'tnt':
            bs.playSound(factory.getRandomExplodeSound(),position=p)
            def _extraBoom():
                bs.playSound(factory.getRandomExplodeSound(),position=p)
            bs.gameTimer(250,_extraBoom)
            def _extraDebrisSound():
                bs.playSound(factory.debrisFallSound,position=p)
                bs.playSound(factory.woodDebrisFallSound,position=p)
            bs.gameTimer(400,_extraDebrisSound)

    def handleMessage(self,message):
        if isinstance(message,bs.DieMessage):
            self.node.delete()

        elif isinstance(message,ExplodeHitMessage):
            node = bs.getCollideValues("opposingNode")
            if node is not None:
                t = self.node.translate

                # new
                mag = 2000.0
                if self.type == 'ice': mag *= 0.5
                elif self.type == 'landMine': mag *= 2.5
                elif self.type == 'tnt': mag *= 2.0

                node.message(bs.HitMessage(pos=t,
                                           velocity=(0,0,0),
                                           magnitude=mag,
                                           hitType=self.hitType,
                                           hitSubType=self.hitSubType,
                                           radius=self.radius,
                                           source=self.source))
                if self.type == "ice":
                    bs.playSound(getFactory().freezeSound,10,position=t)
                    node.message(bs.FreezeMessage())

        else:
            bs.Actor.handleMessage(self,message)

class Bomb(bs.Actor):

    def __init__(self,activity,retain,translate=(0,1,0),velocity=(0,0,0),bombType='normal',blastRadius=2.0,source=None,owner=-1):
        bs.Actor.__init__(self,activity,retain)

        factory = getFactory()

        if not bombType in ('ice','impact','landMine','normal','sticky','tnt'): raise Exception("invalid bomb type: " + bombType)
        self.type = bombType

        #self._activity = weakref.ref(activity)
        self._exploded = False

        if self.type == 'sticky': self._lastStickySoundTime = 0

        self.blastRadius = blastRadius
        if self.type == 'ice': self.blastRadius *= 1.2
        elif self.type == 'impact': self.blastRadius *= 0.7
        elif self.type == 'landMine': self.blastRadius *= 0.7
        elif self.type == 'tnt': self.blastRadius *= 1.45

        # the player this came from
        self.source = source

        # by default our hit type/subtype is our own, but we pick up types of whoever
        # sets us off so we know what caused a chain reaction
        self.hitType = 'explosion'
        self.hitSubType = self.type

        # the node this came from
        self.owner = owner

        # adding footing-materials to things can screw up jumping and flying since players carrying those things
        # and thus touching footing objects will think they're on solid ground..
        # perhaps we don't wanna add this even in the tnt case?..
        if self.type == 'tnt':
            materials = (factory.bombMaterial, bs.footingMaterial, bs.objectMaterial)
        else:
            materials = (factory.bombMaterial, bs.objectMaterial)
            
        if self.type == 'impact': materials = materials + (factory.impactBlastMaterial,)
        elif self.type == 'landMine': materials = materials + (factory.landMineMaterial,)

        if self.type == 'sticky': materials = materials + (factory.stickyMaterial,)
        else: materials = materials + (factory.normalSoundMaterial,)

        if self.type == 'landMine':
            self.node = bs.newNode("prop",
                                   owner=activity,
                                   attrs={'translate':translate,
                                          'velocity':velocity,
                                          'model':factory.landMineModel,
                                          'lightModel':factory.landMineModel,
                                          'body':"landMine",
                                          'shadowSize':0.44,
                                          'colorTexture':factory.landMineTex,
                                          'reflection':"powerup",
                                          'reflectionScale':[1.0],
                                          'materials':materials},
                                   delegate=self,
                                   name="Land Mine")

        elif self.type == 'tnt':
            self.node = bs.newNode("prop",
                                   owner=activity,
                                   attrs={'translate':translate,
                                          'velocity':velocity,
                                          'model':factory.tntModel,
                                          'lightModel':factory.tntModel,
                                          'body':"crate",
                                          'shadowSize':0.5,
                                          'colorTexture':factory.tntTex,
                                          'reflection':"soft",
                                          'reflectionScale':[0.23],
                                          'materials':materials},
                                   delegate=self,name='TNT Box')
            
        elif self.type == 'impact':
            fuseTime = 20000
            self.node = bs.newNode("prop",
                                   owner=activity,
                                   attrs={'translate':translate,
                                          'velocity':velocity,
                                          'model':factory.impactBombModel,
                                          'shadowSize':0.3,
                                          'colorTexture':factory.impactTex,
                                          'reflection':"powerup",
                                          'reflectionScale':[1.5],
                                          'materials':materials},
                                   delegate=self,
                                   name='Impact Bomb')
            self.armTimer = bs.Timer(200,bs.WeakCall(self.handleMessage,ArmMessage()))
            self.warnTimer = bs.Timer(fuseTime-1700,bs.WeakCall(self.handleMessage,WarnMessage()))

        else:
            fuseTime = 3000
            if self.type == 'sticky':
                sticky = True
                scale = 1.0
                model = factory.stickyBombModel
                rType = 'sharper'
                rScale = 1.8
            else:
                sticky = False
                scale = 1.0
                model = factory.bombModel
                rType = 'sharper'
                rScale = 1.8
            if self.type == 'ice': tex = factory.iceTex
            elif self.type == 'sticky': tex = factory.stickyTex
            else: tex = factory.regularTex
            self.node = bs.newNode("bomb",
                                   name='Bomb',
                                   owner=activity,
                                   delegate=self,
                                   attrs={'translate':translate,
                                          'velocity':velocity,
                                          'model':model,
                                          'shadowSize':0.3*scale,
                                          'colorTexture':tex,
                                          'scale':scale,
                                          'sticky':sticky,
                                          'owner':owner,
                                          'reflection':rType,
                                          'reflectionScale':[rScale],
                                          'materials':materials})

            sound = bs.newNode("sound",owner=self.node,attrs={"sound":factory.fuseSound,"volume":0.25})
            self.node.connectAttr("translate",sound,"translate")
            bsUtils.animate(self.node,"fuseLength",{0:1,fuseTime:0})

        # light the fuse!!!
        if self.type not in ('landMine','tnt'):
            bs.gameTimer(fuseTime,bs.WeakCall(self.handleMessage,ExplodeMessage()))

        bsUtils.animate(self.node,"modelScale",{0:0, 200:1.3, 260:1})

    def __handleDie(self,m):
        self.node.delete()
        
    def __handleOOB(self,m):
        self.handleMessage(bs.DieMessage())

    def __handleImpact(self,m):
        node,body = bs.getCollideValues("opposingNode","opposingBody")
        # if we're an impact bomb and we came from this node, don't explode...
        # alternately if we're hitting another impact-bomb from the same source, don't explode...
        try: nodeDelegate = node.getDelegate()
        except Exception: nodeDelegate = None
        if node.exists():
            if (self.type == 'impact' and
                (node == self.owner or (isinstance(nodeDelegate,Bomb) and nodeDelegate.type == 'impact' and nodeDelegate.owner == self.owner))): return
            else: self.handleMessage(ExplodeMessage())

    def __handleDropped(self,m):
        if self.type == 'landMine':
            self.armTimer = bs.Timer(1250,bs.WeakCall(self.handleMessage,ArmMessage()))

        # once we've thrown a sticky bomb we can stick to it..
        elif self.type == 'sticky':
            def _safeSetAttr(node,attr,value):
                if node.exists(): setattr(node,attr,value)
            bs.gameTimer(250,bs.WeakCall(_safeSetAttr,self.node,'stickToOwner',True))

    def __handleSplat(self,m):
        node = bs.getCollideValues("opposingNode")
        if node != self.owner and bs.getGameTime() - self._lastStickySoundTime > 1000:
            self._lastStickySoundTime = bs.getGameTime()
            bs.playSound(getFactory().stickyImpactSound,2.0,position=(self.node,"translate"))

    def __handleExplode(self,m):
        if self._exploded: return
        self._exploded = True
        activity = self._activity()
        if activity and self.node.exists():
            Blast(activity,bs.Actor.RETAINED,
                  translate=self.node.translate,velocity=self.node.velocity,
                  blastRadius=self.blastRadius,type=self.type,source=self.source,hitType=self.hitType,hitSubType=self.hitSubType)
        # we blew up so we need to go away
        bs.gameTimer(1,bs.WeakCall(self.handleMessage,bs.DieMessage()))

    def __handleWarn(self,m):
        if self.oscillator.exists():
            self.oscillator.rate = 30
            bs.playSound(getFactory().warnSound,0.5,position=(self.node,"translate"))

    def __addMaterial(self,material):
        if not self.node.exists(): return
        materials = self.node.materials
        if not material in materials:
            self.node.materials = materials + (material,)
        
    def __handleArm(self,m):
        if not self.node.exists(): return
        factory = getFactory()
        if self.type == 'landMine':
            self.oscillator = bs.newNode("oscillatorInt16",
                                         owner=self.node,
                                         attrs={"input":(factory.landMineLitTex,
                                                         factory.landMineTex),"rate":50})
            bs.gameTimer(500,self.oscillator.delete)
            # we now make it explodable.
            bs.gameTimer(250,bs.WeakCall(self.__addMaterial,factory.landMineBlastMaterial))
        else:
            self.oscillator = bs.newNode("oscillatorInt16",
                                         owner=self.node,
                                         attrs={"input":(factory.impactLitTex,
                                                         factory.impactTex,
                                                         factory.impactTex),"rate":100})
            #bs.gameTimer(250,(self.__addMaterial,factory.landMineBlastMaterial))
            bs.gameTimer(250,bs.WeakCall(self.__addMaterial,factory.landMineBlastMaterial))
        self.oscillator.connectAttr("output",self.node,"colorTexture")
        bs.playSound(factory.activateSound,0.5,position=(self.node,"translate"))
        
    def __handleHit(self,m):
        isPunch = (m.srcNode != 0 and m.srcNode.getType() == 'spaz')

        # normal bombs are triggered by non-punch impacts..  impact-bombs by all impacts
        if not self._exploded and not isPunch or self.type in ['impact','landMine']:
            # also lets change the owner of the bomb to whoever is setting us off..
            # (this way points for big chain reactions go to the person causing them)
            if m.source not in [None,-1]:
                self.source = m.source

                # also inherit the hit type (if a landmine sets off by a bomb, the credit should go to the mine)
                # the exception is TNT.  TNT always gets credit.
                if self.type != 'tnt':
                    self.hitType = m.hitType
                    self.hitSubType = m.hitSubType

            bs.gameTimer(100+int(random.random()*100),bs.WeakCall(self.handleMessage,ExplodeMessage()))
        self.node.message("impulse",m.pos[0],m.pos[1],m.pos[2],
                          m.velocity[0],m.velocity[1],m.velocity[2],
                          m.magnitude,m.velocityMagnitude,m.radius,0,m.velocity[0],m.velocity[1],m.velocity[2])

        if (m.srcNode != 0):
            pass
            #print 'FIXME HANDLE KICKBACK ON BOMB IMPACT'
            # bs.nodeMessage(m.srcNode,"impulse",m.srcBody,m.pos[0],m.pos[1],m.pos[2],
            #                     -0.5*m.force[0],-0.75*m.force[1],-0.5*m.force[2])
        
    def handleMessage(self,m):
        if isinstance(m,ExplodeMessage): self.__handleExplode(m)
        elif isinstance(m,ImpactMessage): self.__handleImpact(m)
        elif isinstance(m,bs.PickedUpMessage):
            # change our source to whoever just picked us up if there's none.
            if self.source not in [None,-1]:
                self.source = m.node.sourcePlayer
        elif isinstance(m,SplatMessage): self.__handleSplat(m)
        elif isinstance(m,bs.DroppedMessage): self.__handleDropped(m)
        elif isinstance(m,bs.HitMessage): self.__handleHit(m)
        elif isinstance(m,bs.DieMessage): self.__handleDie(m)
        elif isinstance(m,bs.OutOfBoundsMessage): self.__handleOOB(m)
        elif isinstance(m,ArmMessage): self.__handleArm(m)
        elif isinstance(m,WarnMessage): self.__handleWarn(m)
        else: bs.Actor.handleMessage(self,m)

class TNTSpawner(object):
    "Regenerates TNT at a point in space every now and then"
    def __init__(self,activity,position,respawnTime=30000):
        self._activity = weakref.ref(activity)
        self._position = position
        self._tnt = None
        self._update()
        self._updateTimer = bs.Timer(1000,bs.WeakCall(self._update),repeat=True)
        self._respawnTime = int(random.uniform(0.8,1.2)*respawnTime)
        self._waitTime = 0
        
    def _update(self):
        tntAlive = self._tnt != None and self._tnt.node.exists()
        if not tntAlive:
            # respawn if its been long enough.. otherwise just increment our how-long-since-we-died value
            activity = self._activity()
            if activity and (self._tnt is None or self._waitTime >= self._respawnTime):
                self._tnt = Bomb(activity,bs.Actor.UNRETAINED,translate=self._position,bombType='tnt')
                self._waitTime = 0
            else: self._waitTime += 1000
